/*
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

//control_window.cpp
//Written by Cosmin Deaconu <cozzyd@ccrma.stanford.edu> 

/* Implements the ControlWindow class described in control_window.h */ 

#include "control_window.h"
#include <gtkmm/stock.h>
#include <gtkmm/aboutdialog.h>
#include <gtkmm/inputdialog.h>
#include <glibmm/fileutils.h>
#include <dlfcn.h>
#include <gtkmm/spinbutton.h>
#include <gtkmm/checkbutton.h>

#define OPTION_TO_WIDGET(X) (1+3*(X+1)) 

/*Quit the program when we exit the command window... there's probably 
 * a more graceful way to do this */
bool ControlWindow::on_delete_event(GdkEventAny * event) {
	exit(0); 
}

/* Called when an option value changes */
void ControlWindow::updateControl() {
	
	//We need to loop through the options and update the lot of them...
	
		std::vector<PluginOption *> * options = inputWindow->getPlugin()->getOptions(); 
		int len = options->size(); 
		
		for (int i = 0; i < len; i++) {
			
			PluginOption * option = (*options)[i];
			Gtk::Widget * w = addedWidgets[OPTION_TO_WIDGET(i)];
			
			switch (option->getType()) {
				
				case OPTION_SLIDER:
					option->setValue( ((Gtk::HScale *) w)->get_value()); 
					break; 
					
				case OPTION_SPIN:
					option->setValue( ((Gtk::SpinButton *) w)->get_value()); 
					break; 
				
				case OPTION_CHECK:
					option->setValue ( ((Gtk::CheckButton *) w)->get_active()); 
				}					
		}
}

/* Dynamically loads the ith plugin in the plugin list 
 * Unloads the previous plugin if one exists */
void ControlWindow::loadPlugin(int i) {
	
		//Get rid of last one
		if (inputWindow->getPlugin() != NULL) {
			inputWindow->getPlugin()->stop(); 
			destroyFn(inputWindow->getPlugin()); 
			dlclose(oldLoad); 
			
			//Destroy old widgets:
			while ( addedWidgets.size() > 0) {
				Gtk::Widget * w = addedWidgets.back(); 
				addedWidgets.pop_back(); 
				w->hide();			
				delete w;
			}
		}
	
		std::string filename = plugins[i]->dir + plugins[i]->name;

		/* TODO: Change to using gmodule so this can be more cross platform */
  	void * plugin = dlopen(filename.c_str(), RTLD_LAZY); 
    	
		if (!plugin) {
			std::cout << dlerror() << std::endl; 
			exit(1); 
		}
  
  	//Associate load_plugin() and destroy_plugin() with the files' functions
		create_t * load_plugin = (create_t *) dlsym(plugin, "init"); 
		destroy_t * destroy_plugin = (destroy_t *) dlsym(plugin, "destroy"); 
			
		if (!load_plugin || !destroy_plugin) {
			std::cout << dlerror() << std::endl; 
			exit(2); 
		}
		
		//Save the destroy function for future use
		destroyFn = destroy_plugin; 		
		
		// Load the new plugin 			
		TSPlugin * tsp = load_plugin(); 
		oldLoad = plugin; 
		
		
		inputWindow->setPlugin(tsp); //Assign the new plugin to the Input Window
		tsp->start(); //Start audio processing
		tsp -> setVolume(volumeControl.get_value()); //Set new volume
		
		/*----Set up inspector (option) controls---*/
		
		/*This is the name of the plugin*/
		Gtk::Label * pluginName  = new Gtk::Label("Name: "+tsp->getName() +" "+ tsp->getVersion());
		pluginName->set_alignment(0,0); 
		controlBox.pack_start(*pluginName,Gtk::PACK_SHRINK);  
		pluginName->show(); 
		
		addedWidgets.push_back((Gtk::Widget *) pluginName);
		
		/*This is the description */
		 /*FIXME: Make the label wrap and stuff... */
		Gtk::Label * pluginDescription = new Gtk::Label("Description: \n"+tsp->getDescription()); 
		pluginDescription->set_alignment(0,0); 
		pluginDescription->set_width_chars(30); 
		pluginDescription->set_single_line_mode(false); 
		pluginDescription->set_line_wrap(true); 
		controlBox.pack_start(*pluginDescription,Gtk::PACK_SHRINK); 
		pluginDescription->show(); 
		
		addedWidgets.push_back((Gtk::Widget *) pluginDescription);
		
		/*Load the options */
		std::vector<PluginOption *> * options = tsp->getOptions(); 
		int len = options->size(); 
		
		for (int i = 0; i < len; i++) {
			
			PluginOption * option = (*options)[i];
			
			/* Make a horizontal box for each control otherwise it looks bad */ 	
			Gtk::HBox * thisBox = new Gtk::HBox(); 
			addedWidgets.push_back((Gtk::Widget *) thisBox);
			controlBox.pack_start(*thisBox, Gtk::PACK_SHRINK); 
			thisBox->show(); 
			
			/* Get the name of the option and create a label for it. Set the 
			 * tooltip to the description */ 
			 
			Gtk::Label * thisLabel = new Gtk::Label(option->getName()); 
			addedWidgets.push_back((Gtk::Widget *) thisLabel);
			thisLabel->set_tooltip_text(option->getDescription()); 
			thisLabel->show(); 
			thisBox->pack_start(*thisLabel); 
				
			/*Figure out what type of control we need and set it up accordingly */
			/*Everything will be connected to updateControl()*/	
			switch(option->getType()) {
				
				case OPTION_SLIDER: {
						Gtk::HScale * slider = new Gtk::HScale();
						slider->set_range(option->getMin(), option->getMax()); 
						slider->set_value(option->getValue()); 
						slider->signal_value_changed().connect(sigc::mem_fun(*this, & ControlWindow::updateControl)); 
						slider->set_increments(option->getMax()/option->getNumSteps(), 2*option->getMax()/option->getNumSteps()); 
						slider->show(); 
						thisBox->pack_start(*slider); 
						addedWidgets.push_back((Gtk::Widget *) slider);
						break; 
				}
						
				case OPTION_SPIN: {
						Gtk::SpinButton * spinner = new Gtk::SpinButton();
						spinner->set_range(option->getMin(), option->getMax()); 
						spinner->set_value(option->getValue()); 
						spinner->signal_value_changed().connect(sigc::mem_fun(*this, & ControlWindow::updateControl)); 
						spinner->set_increments(option->getMax()/option->getNumSteps(), 2*option->getMax()/option->getNumSteps()); 
						spinner->show(); 
						thisBox->pack_start(*spinner); 
						addedWidgets.push_back((Gtk::Widget *) spinner);
						break; 
				}
						
				case OPTION_CHECK: {
						Gtk::CheckButton * check = new Gtk::CheckButton(); 
						check->set_active((int)option->getValue() == 1); 
						check->signal_toggled().connect(sigc::mem_fun(*this, & ControlWindow::updateControl)); 
						check->show(); 
						thisBox->pack_start(*check); 
						addedWidgets.push_back((Gtk::Widget *) check);
						break; 
				}
						
			}
			controlBox.pack_start(*thisBox); 
		}
}

/* Parses the files in plugins/ and puts them in a dropdown box and the plugins
 * vector. Also, if we encounter the default plugin, we enable it */ 
void ControlWindow::loadPlugins() {
	plugins = *load_plugins(); 
	
	for (int c = 0; c < plugins.size(); c++) {
		PluginFileName* plugin = plugins[c]; 		
		chooser.insert_text(c,plugin->name); 
		if (plugin->name==DEFAULT_PLUGIN) {		
			chooser.set_active(c); 
		} 	
	}
}


/* Advanced Input Options */
void ControlWindow::clickInputButton() {	
	inputDialog.run(); 
}

/* Sets up the About Dialog */ 
void ControlWindow::clickAboutButton() {
	
	Gtk::AboutDialog aboutDialog; 
	aboutDialog.set_icon(Gdk::Pixbuf::create_from_inline(-1,icon)); 
	aboutDialog.set_logo(this->get_icon()); 
	aboutDialog.set_name("TabSynth");
	aboutDialog.set_version(VERSION);
	aboutDialog.set_comments("A program to make sound from a graphics tablet. Written by Cosmin Deaconu in 2009 as a project for Music 420 at Stanford University. "); 
	aboutDialog.run(); 
}


/* Constructor */ 
ControlWindow::ControlWindow(InputWindow * wnd) 
: mainBox(), buttonBox(), controlBox(), volumeBox(), titleBox(), pluginSelectorBox(), 
showInput("Input Devices"), 
showAbout(Gtk::Stock::ABOUT),
logo(Gdk::Pixbuf::create_from_inline(-1,icon)),
volumeControl(),
title(std::string("TabSynth") + std::string(VERSION)),
volumeLabel("Volume:"),
pluginLabel("Plugin:"),
inputDialog(),
chooser(),
controlFrame("Plugin Information:"),
addedWidgets()
{
	oldLoad = NULL; 
	inputWindow = wnd; 

  set_icon(Gdk::Pixbuf::create_from_inline(-1,icon)); 
  set_title("TabSynth Control Window");
  set_border_width(5); 
	add(mainBox); 
	mainBox.set_spacing(3); 
	
	
	//Title Box
	mainBox.pack_start(titleBox, Gtk::PACK_SHRINK);
	titleBox.pack_start(logo, Gtk::PACK_SHRINK); 	
	logo.show(); 
	title.set_markup(std::string("<span size=\"x-large\">TabSynth ") + 
		std::string(VERSION) + std::string(" </span>")); 
	titleBox.pack_start(title); 
	title.show(); 
	titleBox.show();
	
	
	//Button Box
	mainBox.pack_start(buttonBox, Gtk::PACK_SHRINK); 
	showInput.signal_clicked().connect(sigc::mem_fun(*this, & ControlWindow::clickInputButton)); 
	showAbout.signal_clicked().connect(sigc::mem_fun(*this, & ControlWindow::clickAboutButton)); 
	buttonBox.pack_start(showInput);
	buttonBox.pack_start(showAbout);
	showInput.show(); 
	showAbout.show(); 
	buttonBox.show(); 
	
	
	//Volume Control
	mainBox.pack_start(volumeBox, Gtk::PACK_SHRINK); 
	volumeBox.pack_start(volumeLabel, Gtk::PACK_SHRINK); 
	volumeLabel.show(); 
	volumeControl.set_range(0,1); 
	volumeControl.set_value(1); 
	volumeControl.set_draw_value(false); 
	volumeControl.signal_value_changed().connect(sigc::mem_fun(*this, & ControlWindow::volumeChanged)); 
	volumeBox.pack_start(volumeControl); 
	volumeControl.show(); 
	volumeBox.show(); 
	
	//Plugin Selector
	mainBox.pack_start(pluginSelectorBox,Gtk::PACK_SHRINK); 
	pluginSelectorBox.pack_start(pluginLabel, Gtk::PACK_SHRINK);
	pluginLabel.show(); 
	pluginSelectorBox.pack_start(chooser); 
	chooser.signal_changed().connect(sigc::mem_fun(*this, & ControlWindow::pluginSelected)); 
	chooser.show(); 
	pluginSelectorBox.show(); 
	
	//Set up the Extended Input dialog
	inputDialog.set_icon(Gdk::Pixbuf::create_from_inline(-1,icon)); 
	inputDialog.get_save_button()->hide(); 
	inputDialog.get_close_button()->signal_clicked().connect
				(sigc::mem_fun(*this,& ControlWindow::hideInput)); 
	
	//Set up control frame
	mainBox.pack_start(controlFrame); 
	controlFrame.add(controlBox); 	
	controlBox.set_spacing(3); 
	controlBox.set_border_width(3); 
	controlBox.show(); 
	controlFrame.show(); 
	mainBox.show(); 

	//Load the plugins
	loadPlugins(); 	
}

/* Hides the extended input dialog */ 
void ControlWindow::hideInput()
{
	inputDialog.hide(); 
}

/*Loads the plugin selected in the drop down box */
void ControlWindow::pluginSelected() {
	loadPlugin(chooser.get_active_row_number());
}

/*Called when the volume slider changes */
void ControlWindow::volumeChanged() {
	inputWindow->getPlugin()->setVolume(volumeControl.get_value()); 
}

/*Really there should be something here, but since the program exits
 * when the window is closed, it seems rather useles... */
ControlWindow::~ControlWindow() {
}
