/*
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

//input_window.cpp
//Written by Cosmin Deaconu <cozzyd@ccrma.stanford.edu> 

/*
 * This file implements the InputWindow class defined in input_window.h 
 */

#include "input_window.h"
#include <iostream>
#include <gdkmm/color.h>
#include <gdkmm/event.h>
#include <gdkmm/device.h>
#include <gdkmm/display.h>

#include <glibmm/listhandle.h>
#include <glibmm/refptr.h>
#include <gtkmm/messagedialog.h>
#include <vector>
#include <sstream>


/* Sets the current plugin of the input window */ 
void InputWindow::setPlugin(TSPlugin * p) {
 	plugin = p;  
 	frame.set_label(p->getName()); 
 	drawingArea.hide();
 	drawingArea.show(); 
}

/* Enables / Disables the status bar */ 
void InputWindow::enableStatusBar(bool enable) {
  statusEnabled = enable; 
}

/* Something changed... so we should tell the plugin about it */

void InputWindow::eventHappened(GdkEventButton * event) {
  gdouble x,y;
  double pressure; 
  bool button[3]; 
  gdouble tiltX,tiltY;
  
  /* Convert to relative coordinates */ 
  x = ((double) event->x) / drawingArea.get_width();
  y = ((double) event->y) / drawingArea.get_height();
  
  
  Gdk::Event ev((GdkEvent*)event); 

  /* Attempt to grab pressure and tilt information */ 
  bool pressureExists = ev.get_axis(Gdk::AXIS_PRESSURE,pressure);
  bool tiltXExists = ev.get_axis(Gdk::AXIS_XTILT,tiltX);
  bool tiltYExists = ev.get_axis(Gdk::AXIS_XTILT,tiltY);
  
  /* If no tilt, set to 0 */
  if (!tiltXExists)
    tiltX = 0;
  if (!tiltYExists)
    tiltY = 0; 
  
  /* Grab mouse button status */
  button[0] = event->state & GDK_BUTTON1_MASK;  
  button[1] = event->state & GDK_BUTTON2_MASK;
  button[2] = event->state & GDK_BUTTON3_MASK;
  
  /* If it's a release or press event, then I think the button mask doesn't
   * work quite right... but that's fine since we have that info */
  if (event->type == GDK_BUTTON_PRESS) {
    if (event->button <= 3)
      button[event->button-1] = 1; 
  } else if (event->type == GDK_BUTTON_RELEASE) {
    if (event->button <= 3)
      button[event->button-1] = 0;   
  }
  
  /* If no pressure, use left mouse button as binary pressure indicator */
  if (!pressureExists)
    pressure = button[0]; 
  
  double freq = 0;
  
  /* If no plugin enabled, output coords and pressure to standard out */ 
  if (plugin==NULL)
  	std::cout << x << " , " << y << " , " << pressure << std::endl; 
  	
  /* Pass the new information to the plugin and retrieve the frequency played */
  else 
  	freq = plugin->setParameters(x,y,pressure, button[1],button[2] ,0,0); 
  
  /* Fill in status bar */
   if (statusEnabled) {
     
     std::ostringstream status1; 
     std::ostringstream status2; 
     std::ostringstream status3; 
     std::ostringstream status4; 
     status1 << "x: " << x; 
     status2 << "y: " << y;
     status3 << "press: " << pressure;
     status4 << "freq:" << freq << " hz"; 
     bar1.pop(); 
     bar2.pop(); 
     bar3.pop(); 
     bar4.pop(); 
     bar1.push(status1.str()); 
     bar2.push(status2.str()); 
     bar3.push(status3.str()); 
     bar4.push(status4.str()); 
  }  
} 


/* This accepts button presses and releases */
bool InputWindow::buttonPressed(GdkEventButton * button) {
  eventHappened( button);
  return true; 
}

/* This accepts motion event signals */
bool InputWindow::motionHappened(GdkEventMotion * event) {
   
  /* The GdkEventButton and GdkEventMotion structs happen to be almost identical,
   * so we can do this pretty nasty-looking cast */
  eventHappened((GdkEventButton *)event);
  return true; 
}

/* Returns the currently enabled plugin */
TSPlugin * InputWindow::getPlugin() {
  return plugin; 
}

/* Called on expose for the plugin. We need to rerun the plugins' draw method*/
bool InputWindow::exposeHappened(GdkEventExpose * event) {
  plugin->draw(drawingArea.get_window()->create_cairo_context(), 
               drawingArea.get_allocation().get_width());   
}; 

InputWindow::InputWindow() 
: drawingArea(), frame(), box(), statusbox(),
bar1(), bar2(), bar3(), bar4()
{
  
  plugin = NULL; 
  set_icon(Gdk::Pixbuf::create_from_inline(-1,icon)); 
  set_title("TabSynth InputWindow");
  set_border_width(4); 
  
  /* Enable signal masks to allow us to listen to the events we want */
  drawingArea.set_extension_events(Gdk::EXTENSION_EVENTS_CURSOR); 
  drawingArea.add_events(Gdk::POINTER_MOTION_MASK); 
  drawingArea.add_events(Gdk::BUTTON_PRESS_MASK); 
  drawingArea.add_events(Gdk::BUTTON_RELEASE_MASK); 
  
  /*Connect signals to methods */
  drawingArea.signal_motion_notify_event().connect(
            sigc::mem_fun(*this, &InputWindow::motionHappened));  
            
  drawingArea.signal_button_press_event().connect(
            sigc::mem_fun(*this, &InputWindow::buttonPressed));
            
  drawingArea.signal_button_release_event().connect(
            sigc::mem_fun(*this, &InputWindow::buttonPressed));

  drawingArea.signal_expose_event().connect(
            sigc::mem_fun(*this, &InputWindow::exposeHappened)); 
            
  drawingArea.set_size_request(DEFAULT_SIZE,DEFAULT_SIZE);
  
  /*Change the background color of the drawing area to white*/ 
  Gdk::Color white("White"); 
  drawingArea.modify_bg(Gtk::STATE_NORMAL, white); 
  

  /*Set up the drawing region*/
  add(box);
  box.pack_start(frame); 
  frame.add(drawingArea); 
  frame.show(); 
  drawingArea.show(); 
  
  
  //Set up the status bar area
  box.pack_start(statusbox,Gtk::PACK_SHRINK); 
  statusbox.pack_start(bar1);
  statusbox.pack_start(bar2);
  statusbox.pack_start(bar3);
  statusbox.pack_start(bar4);
  bar1.show(); 
  bar2.show(); 
  bar3.show(); 
  bar4.show(); 
  statusbox.show(); 
  box.show(); 
  statusEnabled = true; 
    
}

InputWindow::~InputWindow() {}


