//flutemin.cpp
//Written by Cosmin Deaconu <cozzyd@ccrma.stanford.edu> 


#include <math.h>
#include <stk/Flute.h>
#include "../tabsynth_plugin.h"

/* A toy theremin-like instrument that demonsrates how one might
 * perform NoteOn and NoteOff operations. Also, demos all the available
 * option type.
 * 
 * See documentation for more details. 
 *  */

class Flutemin : public TSPlugin {  
  
  protected:
    Flute * flute; //The main instrument
    double vol; 
    bool on; 
    
  public: 
    //Constructor
    Flutemin() {
      flute = new Flute(10); 
      flute->noteOn(10,1.0); 
      vol = 0; 
      on = false; 
      
      //Set up user-settable options
      options.push_back(new PluginOption(OPTION_SLIDER, "Max Freq:", 
                        "The maximum possible frequency", 2000, 1000, 5000, 20)); 
      options.push_back(new PluginOption(OPTION_SPIN, "Exponent:", 
                        "The exponent in the decay", 4,1,10,10)); 
      options.push_back(new PluginOption(OPTION_CHECK, "Double Frequency?", 
                        "Double the frequency", 0)); 
    }
    
    ~Flutemin() {
      delete flute; 
    }
    
    StkFloat setParameters(double x, double y, double pressure, 
                            bool button2, bool button3, double tiltX, 
                            double tiltY) {
      
      double r_x = x - 0.5; 
      if (r_x <0) r_x = -r_x; 
      double r_y = y - 0.5;
      if (r_y <0) r_y = -r_y;
      double r_squared = (r_x*r_x + r_y*r_y)*sqrt(2); 
    
    
      //use a toy model to calculate frequency. The exponent is settable
      //as is the maximum frequency. The "double" option is mostly provided
      //as an example of using an OPTION_CHECK
      double max = options[0]->getValue(); 
      double exp = options[1]->getValue(); 
      double factor = 1 + options[2]->getValue(); 
      double freq = factor * max * pow(( 1 - r_squared ),exp); 
            
      flute->setFrequency(freq); 
      vol = pressure; 
      
      //Note on if we detect pressure >0 and not on already
      if(!on && pressure > 0) {
        on = true; 
        flute-> noteOn(freq,1.0); 
      }
      
      //Note off if we detect pressure <0 and on already
      if (on && pressure == 0) {
        on = false; 
        flute->noteOff(0.5); 
      }
      
      return freq; 
    }

    StkFloat tick()  {
      return flute->tick() * vol; 
    }
    
    /* Draw some circles to make it easier to see what's going on */
    void draw(Cairo::RefPtr<Cairo::Context> cr, int size) {
    
      int c = size/2; 
      
      cr->set_line_width(size * 0.01); 
      cr->set_source_rgb(0,0,1); 
      cr->arc(c,c,1,0,2*M_PI); 
      cr->close_path(); 
      cr->fill_preserve(); 
      cr->stroke(); 
      
      cr->set_line_width(size * 0.005); 
      cr->set_source_rgb(1,0,0); 
      cr->arc(c,c,0.5,0,2*M_PI); 
      cr->close_path(); 
      cr->fill_preserve(); 
      cr->stroke(); 
      
    }
   
    std::string  getName() {
      return "Flutemin"; 
    }
    std::string  getDescription() {
      return "A toy theremin with a flute-y sound"; 
    }

    std::string getVersion() {
      return "0.1"; 
    }
};

/* In addition to containing a class inheriting from TSPlugin, 
 * a plugin must have the following two methods. */

extern "C" TSPlugin * init() {
  return new Flutemin; 
}

extern "C" void destroy(TSPlugin * p) {
  delete p; 
}
