//theremin.cpp
//Written by Cosmin Deaconu <cozzyd@ccrma.stanford.edu> 

#include <cmath>
#include <stk/SineWave.h>
#include <stk/OnePole.h>
#include "../tabsynth_plugin.h"

#define D 0.01                //The diameter of the antenna
#define k 0.4                 //Empirical factor related to ground distance
#define PI 3.14159265358979   //Pi...
#define E0 8.854e-12          //Permittivity of Free Space

#define SMOOTHER_Z 0.99

/* Implements a Theremin plugin with a bunch of tunable parameters. 
 * 
 * See http://adsabs.harvard.edu/abs/1998AmJPh..66..945S for original ref
 * or report.pdf in docs/ for an explanation. 
 * 
 *  */ 


class Theremin : public TSPlugin {  
  
  protected:
    SineWave * wave; // Used to make the sound
    SineWave * tremolor; // Adds tremolo
    OnePole * smoother; 
    double vol; 
    bool trem; 
    
  public: 
  
    //Constructor
    Theremin() {
      wave = new SineWave(); 
      tremolor = new SineWave();      
      smoother = new OnePole(SMOOTHER_Z); 
      vol = 0; 
      trem = false; 
      
      options.push_back(new PluginOption(OPTION_SLIDER, 
                        "L(mH): ", "The Inductance",1, 0.1, 10)); 
      options.push_back(new PluginOption(OPTION_SLIDER, 
                        "C1(pF): ", "The First Capacitance",100, 1, 500)); 
      options.push_back(new PluginOption(OPTION_SLIDER, 
                        "C2(pF): ", "The Second Capacitance",33, 1, 500)); 
      options.push_back(new PluginOption(OPTION_SLIDER, 
                        "h (m): ", "The antenna height", 0.5, 0.1, 2)); 
      options.push_back(new PluginOption(OPTION_SLIDER,
                        "K: ", "The hand size factor", 10, 1, 100)); 
      options.push_back(new PluginOption(OPTION_SLIDER, 
                        "trem: ", "Tremolo frequency (right button toggles)", 
                        1, 0.1,20, 100));
      
    }
    
    ~Theremin() {
      delete wave; 
      delete tremolor; 
    }
    
    StkFloat setParameters(double x, double y, double pressure, 
                           bool button2, bool button3, double tiltX, 
                           double tiltY) {
                              
      double r_x = x - 0.5; 
      if (r_x <0) r_x = -r_x; 
      double r_y = y - 0.5;
      if (r_y <0) r_y = -r_y;

      /* Grab parameters*/ 
      double L = 0.001 * options[0]->getValue(); 
      double c1 = 1e-12 * options[1]->getValue(); 
      double c2 = 1e-12 * options[2]->getValue(); 
      double h = options[3]->getValue(); 
      double K = options[4]->getValue(); 
      
      double f_t = options[5]->getValue(); 
      tremolor->setFrequency(f_t); 
      trem = button3;       
      
      double r_squared =  (r_x*r_x + r_y*r_y);
      x = 2 * sqrt(r_squared);
      if (x < D) x = D; 
      
      //TODO: These calculations can probably be simplified...
      //Capacitance of antenna'd system
      
      double cCommon = 2 * PI * E0 * h / ( log(2*h/D) - k);
      double pi_E0_h_K = PI * E0 * h / K; 
      
      double cA = cCommon + pi_E0_h_K / log(4*x/D);
      
      //Capacitance of "static" system (tuned so that it equals 
      //antenna'd system at 1 m = edge of screen)
      double cA_end = cCommon + pi_E0_h_K / log(4/D);
      
      //Calculate frequencies corresponding to capacitances
      double freq2 =  1./(2*PI*sqrt(L)) * sqrt(1./c1 + 1./(c2+cA_end));
      double freq1 =  1./(2*PI*sqrt(L)) * sqrt(1./c1 + 1./(c2+cA));
            
      //Output the beat frequencies
      double freq = freq2 - freq1; 
      
      //No freq < 0
      freq = freq > 0 ? freq : 0; 
      
      wave->setFrequency(freq); 
      vol = pressure ; 
      return freq; 
      
    }

    StkFloat tick()  {
      //If tremolo is enabled, modulate amplitude w/ tremolor
      if (trem)
        return tremolor->tick() * wave->tick() * smoother->tick(vol); 
      else //otherwise don't 
        return wave->tick() * smoother->tick(vol); 
    }

      /* Draw some circles to make it easier to see what's going on */
    void draw(Cairo::RefPtr<Cairo::Context> cr, int size) {
    
      int c = size/2; 

      //Draws blue part of the center
      cr->set_line_width(size * 0.01); 
      cr->set_source_rgb(0,0,1); 
      cr->arc(c,c,1,0,2*M_PI); 
      cr->close_path(); 
      cr->fill_preserve(); 
      cr->stroke(); 
      
      //Draws red part of the center
      cr->set_line_width(size * 0.005); 
      cr->set_source_rgb(1,0,0); 
      cr->arc(c,c,0.5,0,2*M_PI); 
      cr->close_path(); 
      cr->fill_preserve(); 
      cr->stroke(); 
      
      //Draws outer blue circle
      cr->set_line_width(size * 0.005); 
      cr->set_source_rgb(0,0,1); 
      cr->arc(c,c,c,0,2*M_PI); 
      cr->close_path(); 
      cr->stroke(); 
    }
   
    std::string getName() {
      return "Theremin"; 
    }
    std::string getDescription() {
      return "A physical Theremin model."; 
    }

    std::string getVersion() {
      return "0.1.1"; 
    }
};

/* In addition to containing a class inheriting from TSPlugin, 
 * a plugin must have the following two methods. */

extern "C" TSPlugin * init() {
  return new Theremin; 
}

extern "C" void destroy(TSPlugin * p) {
  delete p; 
}
