/*
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

//tabsynth_plugin.h
//Written by Cosmin Deaconu <cozzyd@ccrma.stanford.edu>  

/* Plugins to TabSynth must extend from this class. This class is responsible
 * for audio synthesis. */


#ifndef TABSYNTH_PLUGIN_H
#define TABSYNTH_PLUGIN_H


#include <cairomm/context.h>
#include <string>
#include <glibmm/thread.h>
#include <stk/RtWvOut.h>
#include <stk/OnePole.h>
#include <vector> 
#include "plugin_option.h"



#define NUM_RAWWAVE_PATHS 4
const static std::string RAWWAVE_PATHS[] = {
  "/usr/local/share/stk/rawwaves/",
  "/usr/share/stk/rawwaves/",
  "./rawwaves/",
  "~/stk/rawwaves/"
  };


#define SAMPLERATE 44100.
#define SMOOTH_Z 0.9 //Defines the pole of the volume smoothing one-pole. 

class TSPlugin 
{
  
  public:
    TSPlugin(); 
    ~TSPlugin(); 
    
    /*Starts audio processing in its own thread*/
    void start(); 
    
    /*Stops audio processing */
    void stop(); 
    
    /* Called by the InputWindow whenever a GTK motion event is triggered
     * in the input area. Should return the resulting frequency. */ 
    virtual StkFloat 
    setParameters(double x,       //x-coordinate of cursor (range: [0,1])
                  double y,       //y-coordinate of cursor (range: [0,1])
                  double pressure,  //stylus pressure (range: [0,1]). If
                                    //no pressure could be found, 0 if no left 
                                    //button, 1 if left button pressed
                                   
                 bool button2,    //Button 2 pressed (middle)
                 bool button3,    //Button 3 presseed (right)
                 double tiltX,    //x-tilt of stylus (NOT YET TESTED) [-1,1]
                 double tiltY);    //y-tilt of stylus (NOT YET TESTED) [-1,1]
  
    /* Sets the global volume of the plugin. This is meant to be 
     * controlled (and kept in synch with) the volume slider in ControlWindow. 
     * It is recommended that a plugin wishing to control its own volume do so 
     * internally */
    void setVolume(double); 
    
    /* Called by the audio processing thread when it needs a new sample. 
     * This how you make sound :) */
    virtual StkFloat tick();        
    
    /* Called by InputWindow on expose for the DrawingArea. Contains a pointer
     * to the cairo context for the DrawingArea and an integer representing the
     * size of the area. */
    virtual void draw(Cairo::RefPtr<Cairo::Context> cairo_context, int size);
    
    /* Should return the name of the plugin. */
    virtual std::string getName();
    
    /* Should return the description of the plugin */
    virtual std::string getDescription(); 
    
    /* Should return the version of the plugin */
    virtual std::string getVersion();
    
    /* Returns the current contents of the options Vector, which should be
     * populated to make user controls. */
    std::vector<PluginOption *> * getOptions();  
    
    protected:
        RtWvOut * out;  
    		bool playing;   
    		void run();     
    		Glib::Thread * thread; 
    		
    		/*This vector should be used to add user controls */
    		std::vector<PluginOption *>  options; 
    		double volume; 
        
        /* Use a one-pole filter to smoothen pressure differences */
        OnePole * smooth; 

};

/* Functions needed for dynamic loading */ 
typedef TSPlugin* create_t(); 
typedef void destroy_t(TSPlugin*); 

#endif
